/* header file */

#ifndef BASE_H

#define BASE_H

// The followings are #define declarations

#define	F_TH	(1e-4)		// Threshhold
#define INFTY	1e20		// Threshhold
#define INF_TH	1e18		// Threshhold

#ifndef PI
#define PI	3.14159
//#define PI05    1.57080
//#define PI2     6.28318
#endif

#define PIHALF  1.57080
#define PIPI    6.28318

#define RGBLIMIT(c)	(unsigned char)(c < 0 ? 0 : (c > 255 ? 255 : c))
#define LIMITRGB(c)     (double)(c < 0.0 ? 0.0 : (c > 1.0 ? 1.0 : c))

// compare function for double types
#define FEQUAL(a,b)	(fabs((a)-(b)) < F_TH)
#define FLARGE(a,b)	((a) > (b) + F_TH)
#define FSMALL(a,b)	((a) < (b) - F_TH)
#define FVAL(a1, a2, b)	\
	(((a1)->y - (a2)->y)*(b)->x + ((a2)->x - (a1)->x)*(b)->y + (a1)->x*(a2)->y - (a2)->x*(a1)->y)
#define DIFFSIGN(x, y)	(((x)<0 && (y)>0) || ((x)>0 && (y)<0))

#ifdef _CONSOLE
typedef int	BOOLEAN;
#define FALSE 0
#define TRUE 1
#endif

typedef double Matrix44[4][4];

typedef struct {
	double x, y, z;
} Vector;

extern Vector *RandomVector();
extern double Length3D(Vector  *v);
extern double DistanceBetween(Vector *,Vector  *);
extern void VNormalize(Vector *v);
extern void VInitialize(Vector **v, double d = 0.0 );
extern void VAdd(Vector  *sum, Vector  *v1, Vector  *v2);
extern void VSub(Vector  *sub, Vector  *v1, Vector  *v2);
extern void VMul(Vector  *mul, Vector  *v1, Vector  *v2);
extern void VDiv(Vector  *div, Vector  *v1, Vector  *v2);
extern void VAdd(Vector  *v1, Vector  *v0, double f);
extern void VSub(Vector  *v1, Vector  *v0, double f);
extern void VDiv(Vector  *v1, Vector  *v0, double f);
extern void VMul(Vector  *v1, Vector  *v0, double f);
extern double VInner(Vector  *v1, Vector  *v2);
extern void VVector(Vector  *x, Vector  *v0, Vector  *v1);
extern BOOLEAN VEqual(Vector *, Vector *);
extern void ClearMatrix(Matrix44 mat);
extern BOOLEAN EqualVector(Vector *v1, Vector *v2);
extern BOOLEAN SmallerVector(Vector *v1, Vector *v2);
extern BOOLEAN GreaterVector(Vector *v1, Vector *v2);
extern BOOLEAN EqualVector(Vector *v1, double f);
extern BOOLEAN SmallerVector(Vector *v1, double f);
extern BOOLEAN GreaterVector(Vector *v1, double f);
extern double VAngle(Vector *, Vector *);

extern void MatrixMatrixMatrix(Matrix44 c, Matrix44 b, Matrix44 a);
extern void InverseMatrix(Matrix44 b, Matrix44 a);
extern void InitializeMatrix(Matrix44 a);
extern void InitializeMatrix(Matrix44 a, double d);
extern void MatrixMatrix(Matrix44 b, Matrix44 a);
extern void VectorMatrix(Vector  *_v, Vector *v0, Matrix44 mat);
extern void CopyMatrix(Matrix44 a, Matrix44 b);
extern void CopyMatrixWithoutPosition(Matrix44 a, Matrix44 b);
extern BOOLEAN EqualMatrix(Matrix44 a, Matrix44 b);
extern void GlobalTranslateMatrix(Matrix44 mat, double x, double y, double z);
extern void LocalTranslateMatrix(Matrix44 mat, double x, double y, double z);
extern void GlobalScaleMatrix(Matrix44 mat, double sx, double sy, double sz);
extern void LocalScaleMatrix(Matrix44 mat, double sx, double sy, double sz);
extern void GlobalRotateMatrix(Matrix44 mat, Vector *v, double th);
extern void LocalRotateMatrix(Matrix44 mat, Vector *v, double th);
extern Vector *PositionMatrix(Matrix44 mat);
extern void  SetPositionMatrix(Matrix44, Vector *);
extern Vector *AngleMatrix(Matrix44 mat);
extern double DistanceBetween(Matrix44, Matrix44);
extern void AddMatrix(Matrix44, Matrix44, double);
extern void SubMatrix(Matrix44, Matrix44, double);
extern void MulMatrix(Matrix44, Matrix44, double);
extern void DivMatrix(Matrix44, Matrix44, double);
extern void MatrixAddMatrix(Matrix44, Matrix44, Matrix44);
extern void MatrixSubMatrix(Matrix44, Matrix44, Matrix44);
extern void MatrixMulMatrix(Matrix44, Matrix44, Matrix44);
extern void MatrixDivMatrix(Matrix44, Matrix44, Matrix44);
extern void ReadMatrix(char *str, Matrix44);
extern void WriteMatrix(char *str, Matrix44);
extern void MakeScaleMatrix(Matrix44 mat, double sx, double sy, double sz);
extern void MakeRotateXMatrix(Matrix44 mat, double thea);
extern void MakeRotateYMatrix(Matrix44 mat, double thea);
extern void MakeRotateZMatrix(Matrix44 mat, double thea);
extern void MakeRotateXYZMatrix(Matrix44 mat, double, double, double);
extern void MakeRotateMatrix(Matrix44 mat, Vector *axis, double theta);
extern void RotateXYZMatrix(Matrix44, Vector *);
extern void MatrixRotateAxisInternal(Matrix44, Vector *, double);
extern void MatrixRotateToInternal(Matrix44, Matrix44,
														Vector *v1 = NULL, double *d1 = NULL,
														Vector *v2 = NULL, double *d2 = NULL);
extern void CopyMatrixWithoutScaling(Matrix44, Matrix44);

extern void TranslateMatrix(Matrix44 mat, Vector *v);
extern void MakeRotateZXYMatrix(Matrix44 mat, double, double, double);
extern void RotateZXYMatrix(Matrix44 mat, Vector *v);

#endif
