// cubeview.cpp : implementation of the CQuickView class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1998 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"

#include <stdio.h>
#include <math.h>

#include "Quick.h"

#include "QuickDoc.h"
#include "QuickView.h"

#include "gl\gl.h"
#include "gl\glu.h"

#include "skeleton.h"
#include "fileio.h"
#include "data.h"
#include "font.h"

#include "InputTextDialog.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

unsigned char threeto8[8] =
{
	0, 0111>>1, 0222>>1, 0333>>1, 0444>>1, 0555>>1, 0666>>1, 0377
};

unsigned char twoto8[4] =
{
	0, 0x55, 0xaa, 0xff
};

unsigned char oneto8[2] =
{
	0, 255
};

int defaultOverride[13] =
{
	0, 3, 24, 27, 64, 67, 88, 173, 181, 236, 247, 164, 91
};

PALETTEENTRY defaultPalEntry[20] =
{
	{ 0,   0,   0,    0 },
	{ 0x80,0,   0,    0 },
	{ 0,   0x80,0,    0 },
	{ 0x80,0x80,0,    0 },
	{ 0,   0,   0x80, 0 },
	{ 0x80,0,   0x80, 0 },
	{ 0,   0x80,0x80, 0 },
	{ 0xC0,0xC0,0xC0, 0 },

	{ 192, 220, 192,  0 },
	{ 166, 202, 240,  0 },
	{ 255, 251, 240,  0 },
	{ 160, 160, 164,  0 },

	{ 0x80,0x80,0x80, 0 },
	{ 0xFF,0,   0,    0 },
	{ 0,   0xFF,0,    0 },
	{ 0xFF,0xFF,0,    0 },
	{ 0,   0,   0xFF, 0 },
	{ 0xFF,0,   0xFF, 0 },
	{ 0,   0xFF,0xFF, 0 },
	{ 0xFF,0xFF,0xFF, 0 }
};

/////////////////////////////////////////////////////////////////////////////
// CQuickView

IMPLEMENT_DYNCREATE(CQuickView, CView)

BEGIN_MESSAGE_MAP(CQuickView, CView)
	//{{AFX_MSG_MAP(CQuickView)
	ON_COMMAND(ID_QUICK_PLAY, OnQuickPlay)
	ON_UPDATE_COMMAND_UI(ID_QUICK_PLAY, OnUpdateQuickPlay)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_TIMER()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_BUTTON32772, OnButton32772)
	ON_UPDATE_COMMAND_UI(ID_BUTTON32772, OnUpdateButton32772)
	ON_COMMAND(ID_BUTTON32773, OnButton32773)
	ON_UPDATE_COMMAND_UI(ID_BUTTON32773, OnUpdateButton32773)
	ON_COMMAND(ID_BUTTON32774, OnButton32774)
	ON_UPDATE_COMMAND_UI(ID_BUTTON32774, OnUpdateButton32774)
	ON_COMMAND(ID_BUTTON32775, OnButton32775)
	ON_UPDATE_COMMAND_UI(ID_BUTTON32775, OnUpdateButton32775)
	ON_COMMAND(ID_BUTTON32776, OnButton32776)
	ON_UPDATE_COMMAND_UI(ID_BUTTON32776, OnUpdateButton32776)
	ON_COMMAND(ID_BUTTON32777, OnButton32777)
	ON_COMMAND(ID_VIEW_VALUES, OnViewValues)
	ON_UPDATE_COMMAND_UI(ID_VIEW_VALUES, OnUpdateViewValues)
	ON_COMMAND(ID_BUTTON32782, OnButton32782)
	ON_COMMAND(ID_BUTTON32783, OnButton32783)
	ON_COMMAND(ID_BUTTON32784, OnButton32784)
	ON_UPDATE_COMMAND_UI(ID_BUTTON32784, OnUpdateButton32784)
	ON_COMMAND(ID_BUTTON32788, OnButton32788)
	ON_UPDATE_COMMAND_UI(ID_BUTTON32788, OnUpdateButton32788)
	ON_COMMAND(IDE_SKT_COLOR, OnSktColor)
	ON_COMMAND(IDE_MESHCOLOR, OnMeshcolor)
	ON_COMMAND(IDE_SS_INTERVAL, OnSsInterval)
	ON_COMMAND(ID_BUTTON32793, OnButton32793)
	ON_UPDATE_COMMAND_UI(ID_BUTTON32793, OnUpdateButton32793)
	ON_COMMAND(IDE_VALUECOLOR, OnValuecolor)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CQuickView construction/destruction

//
// begin by okada on 19/05/2001
//
Vector  eye, center, twist;
double distance;
float	PointSize = 5.0, LineSize = 2.0;
int		SnapShotP;

int 	Width  = 512;
int 	Height = 512;
int     MouseX = 0, dMouseX = 0;
int     MouseY = 0, dMouseY = 0;
int	    dMouseZ = 0;
Vector  MouseVector;
int     MouseState=0;     // true iff left button down
float 	ViewAngle=45.0f;
float	SkeletonColor[3]={ 1.0, 1.0, 1.0 };
float	MeshColor[3]={ 0.5, 0.5, 0.5 };
float	ValuesColor[3]={ 1.0, 1.0, 1.0 };
int		SnapShotInterval = 5;

static void InitializeParameters()
{
    eye.x = 80.0; eye.y = 80.0; eye.z = 400.0;
    center.x = 0.0; center.y = 0.0; center.z = 0.0;
    twist.x = 0.0; twist.y = 1.0; twist.z = 0.0;
	distance = DistanceBetween( &eye, &center );
	VSub( &MouseVector, &eye, &center);
	VNormalize( &MouseVector );
}
//
// end
//

CQuickView *MainView;

CQuickView::CQuickView()
{
	m_pDC = NULL;
	m_pOldPalette = NULL;
	m_play = FALSE;

	MainView = this;
//
// begin by okada on 19/05/2001
//
	RotationModeP = TRUE;
	TranslationModeP = FALSE;
	NoneModeP = FALSE;
	AxisModeP = TRUE;
	PlaneModeP = TRUE;

	GAxisFieldLength = 500.0;	// 500 length
	GAxisFieldDiv = 20;			// 20 div
	TotalGAxisLengthDiv = GAxisFieldDiv * 2;
	GAxisFieldSubLength = GAxisFieldLength / GAxisFieldDiv;

	InitializeParameters();

	construct_skeleton(); // special
	DrawValuesP = FALSE;

//
// end
//
}

CQuickView::~CQuickView()
{
}

/////////////////////////////////////////////////////////////////////////////
// CQuickView drawing

void CQuickView::OnDraw(CDC* pDC)
{
	CQuickDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	DrawScene();
}

/////////////////////////////////////////////////////////////////////////////
// CQuickView diagnostics

#ifdef _DEBUG
void CQuickView::AssertValid() const
{
	CView::AssertValid();
}

void CQuickView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CQuickDoc* CQuickView::GetDocument() // non-debug version is inline
{
	return STATIC_DOWNCAST(CQuickDoc, m_pDocument);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CQuickView message handlers

void CQuickView::OnQuickPlay()
{
	m_play = m_play ? FALSE : TRUE;
	if (m_play)
		SetTimer(1, 15, NULL);
	else
		KillTimer(1);
}

void CQuickView::OnUpdateQuickPlay(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_play);
}

BOOL CQuickView::PreCreateWindow(CREATESTRUCT& cs)
{
	// An OpenGL window must be created with the following flags and must not
	// include CS_PARENTDC for the class style. Refer to SetPixelFormat
	// documentation in the "Comments" section for further information.
	cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

int CQuickView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	Init(); // initialize OpenGL

	return 0;
}

void CQuickView::OnDestroy()
{
//	HGLRC   hrc;

	KillTimer(1);

//	hrc = ::wglGetCurrentContext();

	::wglMakeCurrent(NULL,  NULL);

	if (hrc)
		::wglDeleteContext(hrc);

	if (m_pOldPalette)
		m_pDC->SelectPalette(m_pOldPalette, FALSE);

	if (m_pDC)
		delete m_pDC;

	CView::OnDestroy();
}

void CQuickView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);

	if(cy > 0)
	{
		glViewport(0, 0, cx, cy);

		Width = cx; Height = cy;

/*		if((m_oldRect.right > cx) || (m_oldRect.bottom > cy))
			RedrawWindow();

		m_oldRect.right = cx;
		m_oldRect.bottom = cy;
*/
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(45.0f, (GLdouble)cx/cy, 1.0f, 1000.0f);
		glMatrixMode(GL_MODELVIEW);
	}
}

void CQuickView::OnTimer(UINT nIDEvent)
{
	DrawSkeleton(); // special

	CView::OnTimer(nIDEvent);

	// Eat spurious WM_TIMER messages
	MSG msg;
	while(::PeekMessage(&msg, m_hWnd, WM_TIMER, WM_TIMER, PM_REMOVE));
}

/////////////////////////////////////////////////////////////////////////////
// GL helper functions

void CQuickView::Init()
{
	PIXELFORMATDESCRIPTOR pfd;
	int         n;
	GLfloat     fMaxObjSize, fAspect;
	GLfloat     fNearPlane, fFarPlane;

	m_pDC = new CClientDC(this);

	ASSERT(m_pDC != NULL);

	if (!bSetupPixelFormat())
		return;

	n = ::GetPixelFormat(m_pDC->GetSafeHdc());
	::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

	CreateRGBPalette();

	hrc = wglCreateContext(m_pDC->GetSafeHdc());
	wglMakeCurrent(m_pDC->GetSafeHdc(), hrc);

	GetClientRect(&m_oldRect);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);

	if (m_oldRect.bottom)
		fAspect = (GLfloat)m_oldRect.right/m_oldRect.bottom;
	else    // don't divide by zero, not that we should ever run into that...
		fAspect = 1.0f;
	fNearPlane = 1.0f;
	fFarPlane = 1000.0f;
	fMaxObjSize = 3.0f;
	m_fRadius = fNearPlane + fMaxObjSize / 2.0f;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, fAspect, fNearPlane, fFarPlane);
	glMatrixMode(GL_MODELVIEW);

}

BOOL CQuickView::bSetupPixelFormat()
{
	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
		1,                              // version number
		PFD_DRAW_TO_WINDOW |            // support window
		  PFD_SUPPORT_OPENGL |          // support OpenGL
		  PFD_DOUBLEBUFFER,             // double buffered
		PFD_TYPE_RGBA,                  // RGBA type
		24,                             // 24-bit color depth
		0, 0, 0, 0, 0, 0,               // color bits ignored
		0,                              // no alpha buffer
		0,                              // shift bit ignored
		0,                              // no accumulation buffer
		0, 0, 0, 0,                     // accum bits ignored
		32,                             // 32-bit z-buffer
		0,                              // no stencil buffer
		0,                              // no auxiliary buffer
		PFD_MAIN_PLANE,                 // main layer
		0,                              // reserved
		0, 0, 0                         // layer masks ignored
	};
	int pixelformat;

	if ( (pixelformat = ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)) == 0 )
	{
		MessageBox("ChoosePixelFormat failed");
		return FALSE;
	}

	if (SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd) == FALSE)
	{
		MessageBox("SetPixelFormat failed");
		return FALSE;
	}

	return TRUE;
}

unsigned char CQuickView::ComponentFromIndex(int i, UINT nbits, UINT shift)
{
	unsigned char val;

	val = (unsigned char) (i >> shift);
	switch (nbits)
	{

	case 1:
		val &= 0x1;
		return oneto8[val];
	case 2:
		val &= 0x3;
		return twoto8[val];
	case 3:
		val &= 0x7;
		return threeto8[val];

	default:
		return 0;
	}
}


void CQuickView::CreateRGBPalette()
{
	PIXELFORMATDESCRIPTOR pfd;
	LOGPALETTE *pPal;
	int n, i;

	n = ::GetPixelFormat(m_pDC->GetSafeHdc());
	::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

	if (pfd.dwFlags & PFD_NEED_PALETTE)
	{
		n = 1 << pfd.cColorBits;
		pPal = (PLOGPALETTE) new char[sizeof(LOGPALETTE) + n * sizeof(PALETTEENTRY)];

		ASSERT(pPal != NULL);

		pPal->palVersion = 0x300;
		pPal->palNumEntries = n;
		for (i=0; i<n; i++)
		{
			pPal->palPalEntry[i].peRed =
					ComponentFromIndex(i, pfd.cRedBits, pfd.cRedShift);
			pPal->palPalEntry[i].peGreen =
					ComponentFromIndex(i, pfd.cGreenBits, pfd.cGreenShift);
			pPal->palPalEntry[i].peBlue =
					ComponentFromIndex(i, pfd.cBlueBits, pfd.cBlueShift);
			pPal->palPalEntry[i].peFlags = 0;
		}

		/* fix up the palette to include the default GDI palette */
		if ((pfd.cColorBits == 8)                           &&
			(pfd.cRedBits   == 3) && (pfd.cRedShift   == 0) &&
			(pfd.cGreenBits == 3) && (pfd.cGreenShift == 3) &&
			(pfd.cBlueBits  == 2) && (pfd.cBlueShift  == 6)
		   )
		{
			for (i = 1 ; i <= 12 ; i++)
				pPal->palPalEntry[defaultOverride[i]] = defaultPalEntry[i];
		}

		m_cPalette.CreatePalette(pPal);
		delete pPal;

		m_pOldPalette = m_pDC->SelectPalette(&m_cPalette, FALSE);
		m_pDC->RealizePalette();
	}
}

void CQuickView::DrawAxis()
{
	if ( AxisModeP ){
// draw axis X,Y,Z
		glLineWidth( 2.0 );
		glColor3f(1.0f, 0.0f, 0.0f);
		glBegin(GL_LINES);
			glVertex3f( 100.0, 0.0, 0.0 );
			glVertex3f( 0.0, 0.0, 0.0 );
		glEnd();
		PrintString("X", 100.0, 0.0, 0.0);

		glColor3f(0.0f, 1.0f, 0.0f);
		glBegin(GL_LINES);
			glVertex3f( 0.0, 100.0, 0.0 );
			glVertex3f( 0.0, 0.0, 0.0 );
		glEnd();
		PrintString("Y", 0.0, 100.0, 0.0);

		glColor3f(0.0f, 0.0f, 1.0f);
		glBegin(GL_LINES);
			glVertex3f( 0.0, 0.0, 100.0 );
			glVertex3f( 0.0, 0.0, 0.0 );
		glEnd();
		PrintString("Z", 0.0, 0.0, 100.0);
	}
}

void CQuickView::DrawPlane()
{
	double DAxis[4][3];
	int i, j;

	if ( PlaneModeP ){
		glColor3fv( MeshColor );
		glLineWidth( 1.0 );
		for(i=0;i<2;i++){
			for(j=0;j< TotalGAxisLengthDiv;j++){
				if ( i == 0 ){
					DAxis[i*2][0] = GAxisFieldLength;
					DAxis[i*2][1] = 0.0;
					DAxis[i*2][2] = ( j - GAxisFieldDiv ) * GAxisFieldSubLength;
					DAxis[i*2+1][0] = - GAxisFieldLength;
					DAxis[i*2+1][1] = 0.0;
					DAxis[i*2+1][2] = ( j - GAxisFieldDiv ) * GAxisFieldSubLength;
				} else {
					DAxis[i*2][0] = ( j - GAxisFieldDiv ) * GAxisFieldSubLength;
					DAxis[i*2][1] = 0.0;
					DAxis[i*2][2] = GAxisFieldLength;
					DAxis[i*2+1][0] = ( j - GAxisFieldDiv ) * GAxisFieldSubLength;
					DAxis[i*2+1][1] = 0.0;
					DAxis[i*2+1][2] = - GAxisFieldLength;
				}
				glBegin(GL_LINES);
					glVertex3dv(DAxis[i*2]);
					glVertex3dv(DAxis[i*2+1]);
				glEnd();
			}
		}
	}
}

void CQuickView::DrawModel()
{
	Vector v1;
	unsigned int i;
	char sbuf[16];

	glPushMatrix();

		glPolygonMode(GL_FRONT, GL_POINT);
		glColor3fv( SkeletonColor );
		glPointSize( PointSize );
		glLineWidth( LineSize );

		for(i = 0; i < BVHJoints; i++){
			current_position[i] = *PositionMatrix( skeleton[i].BMat );
			glBegin(GL_POINTS);
			glVertex3d(current_position[i].x, current_position[i].y, current_position[i].z);
			glEnd();
			if ( skeleton[i].parent >= 0 ){
				v1 = *PositionMatrix( skeleton[ skeleton[i].parent ].BMat );
				glBegin(GL_LINES);
				glVertex3d(current_position[i].x, current_position[i].y, current_position[i].z);
				glVertex3d(v1.x, v1.y, v1.z);
				glEnd();
			}
			if ( DrawValuesP && !SnapShotP ){
				sprintf(sbuf,"%d", i);
				PrintString(sbuf, (float)current_position[i].x, (float)current_position[i].y, (float)current_position[i].z);
			}
		}
		if ( DrawValuesP && SnapShotP ){
			sprintf(sbuf, "%d", frame_no);
			PrintString(sbuf, (float)current_position[0].x, (float)current_position[0].y, (float)current_position[0].z);
		}

	glPopMatrix();
}

void CQuickView::DrawValues()
{
	int i;
	static char sbuf[128];

	if ( DrawValuesP && !SnapShotP ){
		glColor3fv( ValuesColor );
		sprintf(sbuf, "Frame no.: %4d", frame_no);
		PrintString(sbuf, 1, 1);
		PrintString("              angle_x    angle_y    angle_z position_x position_y position_z", 1, 2);
		for(i = 0; i < BVHJoints; i++){
			sprintf(sbuf, "joint[%2d]: %10lf %10lf %10lf %10lf %10lf %10lf", i, 
				angles[frame_no][i].x * 180 / PI,
				angles[frame_no][i].y * 180 / PI, angles[frame_no][i].z * 180 / PI,
				current_position[i].x, current_position[i].y, current_position[i].z);
			PrintString(sbuf, 1, i + 3);
		}
	}
}

void CQuickView::DrawScene(void)
{
	if ( hrc == NULL ) return;
	
//	CDC *pdc = GetDC();
//	ASSERT(pdc->m_hDC);
//	wglMakeCurrent(pdc->m_hDC, hrc);

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();

	gluLookAt(eye.x, eye.y, eye.z, center.x, center.y, center.z, twist.x, twist.y, twist.z);

	DrawPlane();
	DrawAxis();

	if ( SnapShotP ){
		for(frame_no = 0; frame_no < NFrames; frame_no += SnapShotInterval){
			update_skeleton( frame_no );
			DrawModel();
		}
	} else DrawModel();

	glPopMatrix();

	DrawValues();

	glFinish();
	SwapBuffers(wglGetCurrentDC());

//	SwapBuffers(pdc->m_hDC);
//	wglMakeCurrent(NULL, NULL);
//    ReleaseDC(pdc);
}

BOOL CQuickView::OnEraseBkgnd(CDC* pDC)
{
	return TRUE;
}

// for chado
void CQuickView::DrawSkeleton()
{
	if ( MotionNo == 1 )
		update_skeleton_of_motion1( frame_no );
	else
		update_skeleton( frame_no );

	frame_no++;
	if ( frame_no >= NFrames ) frame_no = 0;

	DrawScene();
}

void CQuickView::OnFileOpen() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
  char fname[NFILE];
  CFileDialog *pFileDialog = new CFileDialog(TRUE);
  if ( IDCANCEL != pFileDialog->DoModal() ){
	  strcpy(fname , pFileDialog->GetPathName() );
	  if ( readbvh( fname ) ){
		  DrawSkeleton();		  
		  PickupDirectory( CurrentDirectory, fname );
		  PickupFileNameWithoutExtension( CurrentFilename, fname );
//		  MessageBox( CurrentDirectory, CurrentFilename, MB_OK );
	  }
  }
}

void CQuickView::OnFileSaveAs() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
/*
  char fname[NFILE];
  CFileDialog *pFileDialog = new CFileDialog(TRUE);
  if ( IDCANCEL != pFileDialog->DoModal() ){
	  strcpy(fname , pFileDialog->GetPathName() );
	  writetest( fname );
  }
 */
}

void CQuickView::ComputeMouseVector(){
	Matrix44 mat;
	Vector hv, vv, mv;
	double vin;

	if ( RotationModeP ){
		VVector( &hv, &MouseVector, &twist );
		InitializeMatrix( mat );
		MakeRotateMatrix(mat, &hv, - dMouseY * 0.001 );
		VectorMatrix( &mv, &MouseVector, mat );
		VVector( &vv, &hv, &mv );
		MakeRotateMatrix(mat, &vv, dMouseX * 0.001 );
		VectorMatrix( &mv, &mv, mat );
		VNormalize( &mv );
		vin = VInner( &mv, &twist );
		if ( vin < 0.999 && vin > -0.999 ){
			MouseVector.x = mv.x; MouseVector.y = mv.y; MouseVector.z = mv.z;
		}

		distance += dMouseZ * 0.1;
	} else if ( TranslationModeP ){
		VVector( &hv, &MouseVector, &twist );
		VVector( &vv, &hv, &MouseVector );
		InitializeMatrix( mat );
		VMul( &hv, &hv, dMouseX * 0.1 );
		TranslateMatrix(mat, &hv );
		VectorMatrix( &center, &center, mat );
//		VVector( &vv, &hv, &MouseVector );
		VMul( &vv, &vv, dMouseY * 0.1 );
		TranslateMatrix(mat, &vv );
		VectorMatrix( &center, &center, mat );

		distance += dMouseZ * 0.1;
	}
	eye.x = center.x + distance * MouseVector.x;
	eye.y = center.y + distance * MouseVector.y;
	eye.z = center.z + distance * MouseVector.z;
}


void CQuickView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CView::OnLButtonDown(nFlags, point);

		SetCapture();
		MouseX = point.x;
		MouseY = point.y;
		MouseState = 1;
}

void CQuickView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CView::OnLButtonUp(nFlags, point);

	dMouseX = point.x - MouseX;
	dMouseY = point.y - MouseY;
	dMouseZ = 0;
	ComputeMouseVector();
	DrawScene();
	MouseState=0;
		/* remember to release the capture when we are finished. */
	ReleaseCapture();
}

void CQuickView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CView::OnMouseMove(nFlags, point);

	if ( MouseState == 1 ){
		dMouseX = point.x - MouseX;
		dMouseY = point.y - MouseY;
		dMouseZ = 0;
		ComputeMouseVector();
		DrawScene();
	} else if ( MouseState == 2 ){
		dMouseX = 0;
		dMouseY = 0;
		dMouseZ = point.y - MouseY;
		ComputeMouseVector();
		DrawScene();
	}
	MouseX = point.x;
	MouseY = point.y;
}

void CQuickView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CView::OnRButtonDown(nFlags, point);

		SetCapture();
		MouseX = point.x;
		MouseY = point.y;
		MouseState = 2;
}

void CQuickView::OnRButtonUp(UINT nFlags, CPoint point) 
{	
	CView::OnRButtonUp(nFlags, point);

	dMouseX = 0;
	dMouseY = 0;
	dMouseZ = point.y - MouseY;
	ComputeMouseVector();
	DrawScene();
	MouseState=0;
		/* remember to release the capture when we are finished. */
	ReleaseCapture();
}


void CQuickView::OnButton32772() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	RotationModeP = TRUE;
	TranslationModeP = FALSE;
	NoneModeP = FALSE;
}

void CQuickView::OnUpdateButton32772(CCmdUI* pCmdUI) 
{
	// TODO: ̈ʒu command update UI nhp̃R[hǉĂ
	pCmdUI->SetCheck( RotationModeP );		
}

void CQuickView::OnButton32773() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	RotationModeP = FALSE;
	TranslationModeP = TRUE;
	NoneModeP = FALSE;
}

void CQuickView::OnUpdateButton32773(CCmdUI* pCmdUI) 
{
	// TODO: ̈ʒu command update UI nhp̃R[hǉĂ
	pCmdUI->SetCheck( TranslationModeP );			
}


void CQuickView::OnButton32774() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	RotationModeP = FALSE;
	TranslationModeP = FALSE;
	NoneModeP = TRUE;
}

void CQuickView::OnUpdateButton32774(CCmdUI* pCmdUI) 
{
	// TODO: ̈ʒu command update UI nhp̃R[hǉĂ
	pCmdUI->SetCheck( NoneModeP );			
}

void CQuickView::OnButton32775() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	AxisModeP = !AxisModeP;
	DrawScene();
}

void CQuickView::OnUpdateButton32775(CCmdUI* pCmdUI) 
{
	// TODO: ̈ʒu command update UI nhp̃R[hǉĂ
	pCmdUI->SetCheck( AxisModeP );				
}

void CQuickView::OnButton32776() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	PlaneModeP = !PlaneModeP;
	DrawScene();
}

void CQuickView::OnUpdateButton32776(CCmdUI* pCmdUI) 
{
	// TODO: ̈ʒu command update UI nhp̃R[hǉĂ
	pCmdUI->SetCheck(PlaneModeP);	
}

void CQuickView::OnButton32777() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	InitializeParameters();
	DrawScene();
}

void CQuickView::OnViewValues() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	DrawValuesP = !DrawValuesP;
	DrawScene();
}

void CQuickView::OnUpdateViewValues(CCmdUI* pCmdUI) 
{
	// TODO: ̈ʒu command update UI nhp̃R[hǉĂ
	pCmdUI->SetCheck( DrawValuesP );			
}

void CQuickView::OnButton32782() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	update_skeleton( frame_no-- );
	if ( frame_no < 0 ) frame_no = NFrames - 1;
	DrawScene();
}

void CQuickView::OnButton32783() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	update_skeleton( frame_no++ );
	if ( frame_no >= NFrames ) frame_no = 0;
	DrawScene();	
}

void CQuickView::OnButton32784() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	OnQuickPlay();
	if ( SnapShotP && m_play ) SnapShotP = FALSE;
}

void CQuickView::OnUpdateButton32784(CCmdUI* pCmdUI) 
{
	// TODO: ̈ʒu command update UI nhp̃R[hǉĂ
	pCmdUI->SetCheck( m_play );	
}

void CQuickView::OnButton32788() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	SnapShotP = !SnapShotP;
	if ( SnapShotP && m_play ) OnQuickPlay();
	DrawScene();
}

void CQuickView::OnUpdateButton32788(CCmdUI* pCmdUI) 
{
	// TODO: ̈ʒu command update UI nhp̃R[hǉĂ
	pCmdUI->SetCheck( SnapShotP );	
}

void CQuickView::OnSktColor() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	static char s[32];

	sprintf(s, "%3.2f, %3.2f, %3.2f", SkeletonColor[0], SkeletonColor[1], SkeletonColor[2]);
	if ( OpenInputTextDialog(s, "SkeletonColor", "Three floats") ){
		sscanf(s, "%f, %f, %f", &SkeletonColor[0], &SkeletonColor[1], &SkeletonColor[2]);
		DrawScene();
	}	
}

void CQuickView::OnMeshcolor() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	static char s[32];

	sprintf(s, "%3.2f, %3.2f, %3.2f", MeshColor[0], MeshColor[1], MeshColor[2]);
	if ( OpenInputTextDialog(s, "MeshColor", "Three floats") ){
		sscanf(s, "%f, %f, %f", &MeshColor[0], &MeshColor[1], &MeshColor[2]);
		DrawScene();
	}		
}

void CQuickView::OnSsInterval() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	static char s[32];

	sprintf(s, "%3d", SnapShotInterval);
	if ( OpenInputTextDialog(s, "SnapShotInterval", "Enter Integer") ){
		sscanf(s, "%d", &SnapShotInterval);
		DrawScene();
	}			
}

void CQuickView::OnButton32793() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	DrawValuesP = !DrawValuesP;
	DrawScene();	
}

void CQuickView::OnUpdateButton32793(CCmdUI* pCmdUI) 
{
	// TODO: ̈ʒu command update UI nhp̃R[hǉĂ
	pCmdUI->SetCheck( DrawValuesP );	
}

void CQuickView::OnValuecolor() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	static char s[32];

	sprintf(s, "%3.2f, %3.2f, %3.2f", ValuesColor[0], ValuesColor[1], ValuesColor[2]);
	if ( OpenInputTextDialog(s, "ValuesColor", "Three floats") ){
		sscanf(s, "%f, %f, %f", &ValuesColor[0], &ValuesColor[1], &ValuesColor[2]);
		DrawScene();
	}	
}
